@@ -8,58 +8,70 @@ plugin interfaces used by Salt.
 # Import python libs
 from __future__ import absolute_import
 import os
-import imp
 import sys
-import salt
 import time
 import logging
 import inspect
 import tempfile
 import functools
+import types
 from collections import MutableMapping
 from zipimport import zipimporter
 
 # Import salt libs
-from salt.exceptions import LoaderError
-from salt.template import check_render_pipe_str
-from salt.utils.decorators import Depends
-from salt.utils import is_proxy
+import salt.config
+import salt.syspaths
 import salt.utils.context
 import salt.utils.lazy
 import salt.utils.event
 import salt.utils.odict
-
-# Solve the Chicken and egg problem where grains need to run before any
-# of the modules are loaded and are generally available for any usage.
-import salt.modules.cmdmod
+from salt.exceptions import LoaderError
+from salt.template import check_render_pipe_str
+from salt.utils.decorators import Depends
+from salt.utils import is_proxy
 
 # Import 3rd-party libs
 import salt.ext.six as six
+from salt.ext.six.moves import reload_module
+
+if sys.version_info[:2] >= (3, 5):
+    import importlib.machinery  # pylint: disable=no-name-in-module,import-error
+    import importlib.util  # pylint: disable=no-name-in-module,import-error
+    USE_IMPORTLIB = True
+else:
+    import imp
+    USE_IMPORTLIB = False
+
 try:
     import pkg_resources
     HAS_PKG_RESOURCES = True
 except ImportError:
     HAS_PKG_RESOURCES = False
 
-__salt__ = {
-    'cmd.run': salt.modules.cmdmod._run_quiet
-}
 log = logging.getLogger(__name__)
 
-SALT_BASE_PATH = os.path.abspath(os.path.dirname(salt.__file__))
+SALT_BASE_PATH = os.path.abspath(salt.syspaths.INSTALL_DIR)
 LOADED_BASE_NAME = 'salt.loaded'
 
-if six.PY3:
-    # pylint: disable=no-member,no-name-in-module,import-error
-    import importlib.machinery
+if USE_IMPORTLIB:
+    # pylint: disable=no-member
+    MODULE_KIND_SOURCE = 1
+    MODULE_KIND_COMPILED = 2
+    MODULE_KIND_EXTENSION = 3
+    MODULE_KIND_PKG_DIRECTORY = 5
     SUFFIXES = []
     for suffix in importlib.machinery.EXTENSION_SUFFIXES:
-        SUFFIXES.append((suffix, 'rb', 3))
+        SUFFIXES.append((suffix, 'rb', MODULE_KIND_EXTENSION))
     for suffix in importlib.machinery.BYTECODE_SUFFIXES:
-        SUFFIXES.append((suffix, 'rb', 2))
+        SUFFIXES.append((suffix, 'rb', MODULE_KIND_COMPILED))
     for suffix in importlib.machinery.SOURCE_SUFFIXES:
-        SUFFIXES.append((suffix, 'r', 1))
-    # pylint: enable=no-member,no-name-in-module,import-error
+        SUFFIXES.append((suffix, 'rb', MODULE_KIND_SOURCE))
+    MODULE_KIND_MAP = {
+        MODULE_KIND_SOURCE: importlib.machinery.SourceFileLoader,
+        MODULE_KIND_COMPILED: importlib.machinery.SourcelessFileLoader,
+        MODULE_KIND_EXTENSION: importlib.machinery.ExtensionFileLoader
+    }
+    # pylint: enable=no-member
 else:
     SUFFIXES = imp.get_suffixes()
 
@@ -272,13 +284,14 @@ def raw_mod(opts, name, functions, mod='modules'):
     return dict(loader._dict)  # return a copy of *just* the funcs for `name`
 
 
-def engines(opts, functions, runners, proxy=None):
+def engines(opts, functions, runners, utils, proxy=None):
     '''
     Return the master services plugins
     '''
     pack = {'__salt__': functions,
             '__runners__': runners,
-            '__proxy__': proxy}
+            '__proxy__': proxy,
+            '__utils__': utils}
     return LazyLoader(
         _module_dirs(opts, 'engines'),
         opts,
@@ -429,7 +442,7 @@ def fileserver(opts, backends):
                       pack={'__utils__': utils(opts)})
 
 
-def roster(opts, whitelist=None):
+def roster(opts, runner, whitelist=None):
     '''
     Returns the roster modules
     '''
@@ -438,6 +451,7 @@ def roster(opts, whitelist=None):
         opts,
         tag='roster',
         whitelist=whitelist,
+        pack={'__runner__': runner},
     )
 
 
@@ -495,8 +509,8 @@ def beacons(opts, functions, context=None, proxy=None):
         _module_dirs(opts, 'beacons'),
         opts,
         tag='beacons',
-        virtual_funcs=['__validate__'],
         pack={'__context__': context, '__salt__': functions, '__proxy__': proxy or {}},
+        virtual_funcs=['__validate__'],
     )
 
 
@@ -632,6 +646,8 @@ def grains(opts, force_refresh=False, proxy=None):
         __grains__ = salt.loader.grains(__opts__)
         print __grains__['id']
     '''
+    # Need to re-import salt.config, somehow it got lost when a minion is starting
+    import salt.config
     # if we have no grains, lets try loading from disk (TODO: move to decorator?)
     cfn = os.path.join(
         opts['cachedir'],
@@ -652,17 +668,16 @@ def grains(opts, force_refresh=False, proxy=None):
                     except (IOError, OSError):
                         pass
                 else:
-                    if force_refresh:
-                        log.debug('Grains refresh requested. Refreshing grains.')
-                    else:
-                        log.debug('Grains cache last modified {0} seconds ago and '
-                                  'cache expiration is set to {1}. '
-                                  'Grains cache expired. Refreshing.'.format(
-                                      grains_cache_age,
-                                      opts.get('grains_cache_expiration', 300)
-                                  ))
+                    log.debug('Grains cache last modified {0} seconds ago and '
+                              'cache expiration is set to {1}. '
+                              'Grains cache expired. Refreshing.'.format(
+                                  grains_cache_age,
+                                  opts.get('grains_cache_expiration', 300)
+                              ))
             else:
                 log.debug('Grains cache file does not exist.')
+    else:
+        log.debug('Grains refresh requested. Refreshing grains.')
 
     if opts.get('skip_grains', False):
         return {}
@@ -695,11 +710,11 @@ def grains(opts, force_refresh=False, proxy=None):
     if force_refresh:  # if we refresh, lets reload grain modules
         funcs.clear()
     # Run core grains
-    for key, fun in six.iteritems(funcs):
+    for key in funcs:
         if not key.startswith('core.'):
             continue
         log.trace('Loading {0} grain'.format(key))
-        ret = fun()
+        ret = funcs[key]()
         if not isinstance(ret, dict):
             continue
         if grains_deep_merge:
@@ -708,7 +723,7 @@ def grains(opts, force_refresh=False, proxy=None):
             grains_data.update(ret)
 
     # Run the rest of the grains
-    for key, fun in six.iteritems(funcs):
+    for key in funcs:
         if key.startswith('core.') or key == '_errors':
             continue
         try:
@@ -718,17 +733,18 @@ def grains(opts, force_refresh=False, proxy=None):
             # one parameter.  Then the grains can have access to the
             # proxymodule for retrieving information from the connected
             # device.
-            if fun.__code__.co_argcount == 1:
-                ret = fun(proxy)
+            log.trace('Loading {0} grain'.format(key))
+            if funcs[key].__code__.co_argcount == 1:
+                ret = funcs[key](proxy)
             else:
-                ret = fun()
+                ret = funcs[key]()
         except Exception:
             if is_proxy():
                 log.info('The following CRITICAL message may not be an error; the proxy may not be completely established yet.')
             log.critical(
                 'Failed to load grains defined in grain file {0} in '
                 'function {1}, error:\n'.format(
-                    key, fun
+                    key, funcs[key]
                 ),
                 exc_info=True
             )
@@ -740,7 +756,7 @@ def grains(opts, force_refresh=False, proxy=None):
         else:
             grains_data.update(ret)
 
-    if opts.get('proxy_merge_grains_in_module', False) and proxy:
+    if opts.get('proxy_merge_grains_in_module', True) and proxy:
         try:
             proxytype = proxy.opts['proxy']['proxytype']
             if proxytype+'.grains' in proxy:
@@ -765,8 +781,10 @@ def grains(opts, force_refresh=False, proxy=None):
         cumask = os.umask(0o77)
         try:
             if salt.utils.is_windows():
+                # Late import
+                import salt.modules.cmdmod
                 # Make sure cache file isn't read-only
-                __salt__['cmd.run']('attrib -R "{0}"'.format(cfn))
+                salt.modules.cmdmod._run_quiet('attrib -R "{0}"'.format(cfn))
             with salt.utils.fopen(cfn, 'w+b') as fp_:
                 try:
                     serial = salt.payload.Serial(opts)
@@ -831,15 +849,22 @@ def queues(opts):
     )
 
 
-def sdb(opts, functions=None, whitelist=None):
+def sdb(opts, functions=None, whitelist=None, utils=None):
     '''
     Make a very small database call
     '''
+    if utils is None:
+        utils = {}
+
     return LazyLoader(
         _module_dirs(opts, 'sdb'),
         opts,
         tag='sdb',
-        pack={'__sdb__': functions},
+        pack={
+            '__sdb__': functions,
+            '__opts__': opts,
+            '__utils__': utils,
+        },
         whitelist=whitelist,
     )
 
@@ -947,7 +972,7 @@ def _generate_module(name):
         return
 
     code = "'''Salt loaded {0} parent module'''".format(name.split('.')[-1])
-    module = imp.new_module(name)
+    module = types.ModuleType(name)
     exec(code, module.__dict__)
     sys.modules[name] = module
 
@@ -1096,6 +1121,9 @@ class LazyLoader(salt.utils.lazy.LazyDict):
         Allow for "direct" attribute access-- this allows jinja templates to
         access things like `salt.test.ping()`
         '''
+        if mod_name in ('__getstate__', '__setstate__'):
+            return object.__getattribute__(self, mod_name)
+
         # if we have an attribute named that, lets return it.
         try:
             return object.__getattr__(self, mod_name)  # pylint: disable=no-member
@@ -1161,7 +1189,10 @@ class LazyLoader(salt.utils.lazy.LazyDict):
         if self.opts.get('enable_zip_modules', True) is True:
             self.suffix_map['.zip'] = tuple()
         # allow for module dirs
-        self.suffix_map[''] = ('', '', imp.PKG_DIRECTORY)
+        if USE_IMPORTLIB:
+            self.suffix_map[''] = ('', '', MODULE_KIND_PKG_DIRECTORY)
+        else:
+            self.suffix_map[''] = ('', '', imp.PKG_DIRECTORY)
 
         # create mapping of filename (without suffix) to (path, suffix)
         # The files are added in order of priority, so order *must* be retained.
@@ -1170,7 +1201,8 @@ class LazyLoader(salt.utils.lazy.LazyDict):
         for mod_dir in self.module_dirs:
             files = []
             try:
-                files = os.listdir(mod_dir)
+                # Make sure we have a sorted listdir in order to have expectable override results
+                files = sorted(os.listdir(mod_dir))
             except OSError:
                 continue  # Next mod_dir
             for filename in files:
@@ -1286,7 +1318,7 @@ class LazyLoader(salt.utils.lazy.LazyDict):
         for submodule in submodules:
             # it is a submodule if the name is in a namespace under mod
             if submodule.__name__.startswith(mod.__name__ + '.'):
-                reload(submodule)
+                reload_module(submodule)
                 self._reload_submodules(submodule)
 
     def _load_module(self, name):
@@ -1325,26 +1357,72 @@ class LazyLoader(salt.utils.lazy.LazyDict):
                         self.tag,
                         name)
                 if suffix == '':
-                    mod = imp.load_module(mod_namespace, None, fpath, desc)
+                    if USE_IMPORTLIB:
+                        # pylint: disable=no-member
+                        # Package directory, look for __init__
+                        loader_details = [
+                            (importlib.machinery.SourceFileLoader, importlib.machinery.SOURCE_SUFFIXES),
+                            (importlib.machinery.SourcelessFileLoader, importlib.machinery.BYTECODE_SUFFIXES),
+                            (importlib.machinery.ExtensionFileLoader, importlib.machinery.EXTENSION_SUFFIXES),
+                        ]
+                        file_finder = importlib.machinery.FileFinder(
+                            fpath_dirname,
+                            *loader_details
+                        )
+                        spec = file_finder.find_spec(mod_namespace)
+                        if spec is None:
+                            raise ImportError()
+                        # TODO: Get rid of load_module in favor of
+                        # exec_module below. load_module is deprecated, but
+                        # loading using exec_module has been causing odd things
+                        # with the magic dunders we pack into the loaded
+                        # modules, most notably with salt-ssh's __opts__.
+                        mod = spec.loader.load_module()
+                        # mod = importlib.util.module_from_spec(spec)
+                        # spec.loader.exec_module(mod)
+                        # pylint: enable=no-member
+                        sys.modules[mod_namespace] = mod
+                    else:
+                        mod = imp.load_module(mod_namespace, None, fpath, desc)
                     # reload all submodules if necessary
                     if not self.initial_load:
                         self._reload_submodules(mod)
                 else:
-                    with salt.utils.fopen(fpath, desc[1]) as fn_:
-                        mod = imp.load_module(mod_namespace, fn_, fpath, desc)
-
+                    if USE_IMPORTLIB:
+                        # pylint: disable=no-member
+                        loader = MODULE_KIND_MAP[desc[2]](mod_namespace, fpath)
+                        spec = importlib.util.spec_from_file_location(
+                            mod_namespace, fpath, loader=loader
+                        )
+                        if spec is None:
+                            raise ImportError()
+                        # TODO: Get rid of load_module in favor of
+                        # exec_module below. load_module is deprecated, but
+                        # loading using exec_module has been causing odd things
+                        # with the magic dunders we pack into the loaded
+                        # modules, most notably with salt-ssh's __opts__.
+                        mod = spec.loader.load_module()
+                        #mod = importlib.util.module_from_spec(spec)
+                        #spec.loader.exec_module(mod)
+                        # pylint: enable=no-member
+                        sys.modules[mod_namespace] = mod
+                    else:
+                        with salt.utils.fopen(fpath, desc[1]) as fn_:
+                            mod = imp.load_module(mod_namespace, fn_, fpath, desc)
         except IOError:
             raise
         except ImportError as exc:
             if 'magic number' in str(exc):
-                log.warning('Failed to import {0} {1}. Bad magic number. If migrating '
-                        'from Python2 to Python3, remove all .pyc files and try again.'.format(self.tag, name))
+                error_msg = 'Failed to import {0} {1}. Bad magic number. If migrating from Python2 to Python3, remove all .pyc files and try again.'.format(self.tag, name)
+                log.warning(error_msg)
+                self.missing_modules[name] = error_msg
             log.debug(
                 'Failed to import {0} {1}:\n'.format(
                     self.tag, name
                 ),
                 exc_info=True
             )
+            self.missing_modules[name] = exc
             return False
         except Exception as error:
             log.error(
@@ -1354,14 +1432,16 @@ class LazyLoader(salt.utils.lazy.LazyDict):
                 ),
                 exc_info=True
             )
+            self.missing_modules[name] = error
             return False
-        except SystemExit:
+        except SystemExit as error:
             log.error(
                 'Failed to import {0} {1} as the module called exit()\n'.format(
                     self.tag, name
                 ),
                 exc_info=True
             )
+            self.missing_modules[name] = error
             return False
         finally:
             sys.path.remove(fpath_dirname)
@@ -1399,10 +1479,8 @@ class LazyLoader(salt.utils.lazy.LazyDict):
         if self.virtual_enable:
             virtual_funcs_to_process = ['__virtual__'] + self.virtual_funcs
             for virtual_func in virtual_funcs_to_process:
-                (virtual_ret, module_name, virtual_err) = self.process_virtual(
-                    mod,
-                    module_name,
-                )
+                virtual_ret, module_name, virtual_err, virtual_aliases = \
+                    self.process_virtual(mod, module_name)
                 if virtual_err is not None:
                     log.trace(
                         'Error loading %s.%s: %s',
@@ -1416,6 +1494,8 @@ class LazyLoader(salt.utils.lazy.LazyDict):
                     self.missing_modules[module_name] = virtual_err
                     self.missing_modules[name] = virtual_err
                     return False
+        else:
+            virtual_aliases = ()
 
         # If this is a proxy minion then MOST modules cannot work. Therefore, require that
         # any module that does work with salt-proxy-minion define __proxyenabled__ as a list
@@ -1442,17 +1522,18 @@ class LazyLoader(salt.utils.lazy.LazyDict):
 
         # If we had another module by the same virtual name, we should put any
         # new functions under the existing dictionary.
-        if module_name in self.loaded_modules:
-            mod_dict = self.loaded_modules[module_name]
-        else:
-            mod_dict = self.mod_dict_class()
+        mod_names = [module_name] + list(virtual_aliases)
+        mod_dict = dict((
+            (x, self.loaded_modules.get(x, self.mod_dict_class()))
+            for x in mod_names
+        ))
 
         for attr in getattr(mod, '__load__', dir(mod)):
             if attr.startswith('_'):
                 # private functions are skipped
                 continue
             func = getattr(mod, attr)
-            if not inspect.isfunction(func):
+            if not inspect.isfunction(func) and not isinstance(func, functools.partial):
                 # Not a function!? Skip it!!!
                 continue
             # Let's get the function name.
@@ -1463,18 +1544,19 @@ class LazyLoader(salt.utils.lazy.LazyDict):
             # It default's of course to the found callable attribute name
             # if no alias is defined.
             funcname = getattr(mod, '__func_alias__', {}).get(attr, attr)
-            try:
-                full_funcname = '.'.join((module_name, funcname))
-            except TypeError:
-                full_funcname = '{0}.{1}'.format(module_name, funcname)
-            # Save many references for lookups
-            # Careful not to overwrite existing (higher priority) functions
-            if full_funcname not in self._dict:
-                self._dict[full_funcname] = func
-            if funcname not in mod_dict:
-                setattr(mod_dict, funcname, func)
-                mod_dict[funcname] = func
-                self._apply_outputter(func, mod)
+            for tgt_mod in mod_names:
+                try:
+                    full_funcname = '.'.join((tgt_mod, funcname))
+                except TypeError:
+                    full_funcname = '{0}.{1}'.format(tgt_mod, funcname)
+                # Save many references for lookups
+                # Careful not to overwrite existing (higher priority) functions
+                if full_funcname not in self._dict:
+                    self._dict[full_funcname] = func
+                if funcname not in mod_dict[tgt_mod]:
+                    setattr(mod_dict[tgt_mod], funcname, func)
+                    mod_dict[tgt_mod][funcname] = func
+                    self._apply_outputter(func, mod)
 
         # enforce depends
         try:
@@ -1483,7 +1565,8 @@ class LazyLoader(salt.utils.lazy.LazyDict):
             log.info('Depends.enforce_dependencies() failed '
                      'for reasons: {0}'.format(exc))
 
-        self.loaded_modules[module_name] = mod_dict
+        for tgt_mod in mod_names:
+            self.loaded_modules[tgt_mod] = mod_dict[tgt_mod]
         return True
 
     def _load(self, key):
@@ -1541,6 +1624,10 @@ class LazyLoader(salt.utils.lazy.LazyDict):
 
         self.loaded = True
 
+    def reload_modules(self):
+        self.loaded_files = set()
+        self._load_all()
+
     def _apply_outputter(self, func, mod):
         '''
         Apply the __outputter__ variable to the functions
@@ -1576,6 +1663,7 @@ class LazyLoader(salt.utils.lazy.LazyDict):
         # namespace collisions. And finally it allows modules to return False
         # if they are not intended to run on the given platform or are missing
         # dependencies.
+        virtual_aliases = getattr(mod, '__virtual_aliases__', tuple())
         try:
             error_reason = None
             if hasattr(mod, '__virtual__') and inspect.isfunction(mod.__virtual__):
@@ -1591,9 +1679,10 @@ class LazyLoader(salt.utils.lazy.LazyDict):
                                 end, module_name)
                         log.warning(msg)
                 except Exception as exc:
-                    error_reason = ('Exception raised when processing __virtual__ function'
-                              ' for {0}. Module will not be loaded {1}'.format(
-                                  module_name, exc))
+                    error_reason = (
+                        'Exception raised when processing __virtual__ function'
+                        ' for {0}. Module will not be loaded: {1}'.format(
+                            mod.__name__, exc))
                     log.error(error_reason, exc_info_on_loglevel=logging.DEBUG)
                     virtual = None
                 # Get the module's virtual name
@@ -1616,7 +1705,7 @@ class LazyLoader(salt.utils.lazy.LazyDict):
                             )
                         )
 
-                    return (False, module_name, error_reason)
+                    return (False, module_name, error_reason, virtual_aliases)
 
                 # At this point, __virtual__ did not return a
                 # boolean value, let's check for deprecated usage
@@ -1685,9 +1774,9 @@ class LazyLoader(salt.utils.lazy.LazyDict):
                 ),
                 exc_info=True
             )
-            return (False, module_name, error_reason)
+            return (False, module_name, error_reason, virtual_aliases)
 
-        return (True, module_name, None)
+        return (True, module_name, None, virtual_aliases)
 
 
 def global_injector_decorator(inject_globals):
