@@ -10,6 +10,8 @@ import os
 import collections
 import logging
 import tornado.gen
+import sys
+import traceback
 
 # Import salt libs
 import salt.loader
@@ -19,6 +21,7 @@ import salt.crypt
 import salt.transport
 import salt.utils.url
 import salt.utils.cache
+import salt.utils.crypt
 from salt.exceptions import SaltClientError
 from salt.template import compile_template
 from salt.utils.dictupdate import merge
@@ -32,7 +35,7 @@ log = logging.getLogger(__name__)
 
 
 def get_pillar(opts, grains, minion_id, saltenv=None, ext=None, funcs=None,
-               pillar=None, pillarenv=None):
+               pillar_override=None, pillarenv=None):
     '''
     Return the correct pillar driver based on the file_client option
     '''
@@ -49,14 +52,14 @@ def get_pillar(opts, grains, minion_id, saltenv=None, ext=None, funcs=None,
         log.info('Compiling pillar from cache')
         log.debug('get_pillar using pillar cache with ext: {0}'.format(ext))
         return PillarCache(opts, grains, minion_id, saltenv, ext=ext, functions=funcs,
-                pillar=pillar, pillarenv=pillarenv)
+                pillar_override=pillar_override, pillarenv=pillarenv)
     return ptype(opts, grains, minion_id, saltenv, ext, functions=funcs,
-                 pillar=pillar, pillarenv=pillarenv)
+                 pillar_override=pillar_override, pillarenv=pillarenv)
 
 
 # TODO: migrate everyone to this one!
 def get_async_pillar(opts, grains, minion_id, saltenv=None, ext=None, funcs=None,
-               pillar=None, pillarenv=None):
+               pillar_override=None, pillarenv=None):
     '''
     Return the correct pillar driver based on the file_client option
     '''
@@ -68,7 +71,7 @@ def get_async_pillar(opts, grains, minion_id, saltenv=None, ext=None, funcs=None
         'local': AsyncPillar,
     }.get(file_client, AsyncPillar)
     return ptype(opts, grains, minion_id, saltenv, ext, functions=funcs,
-                 pillar=pillar, pillarenv=pillarenv)
+                 pillar_override=pillar_override, pillarenv=pillarenv)
 
 
 class AsyncRemotePillar(object):
@@ -76,21 +79,19 @@ class AsyncRemotePillar(object):
     Get the pillar from the master
     '''
     def __init__(self, opts, grains, minion_id, saltenv, ext=None, functions=None,
-                 pillar=None, pillarenv=None):
+                 pillar_override=None, pillarenv=None):
         self.opts = opts
         self.opts['environment'] = saltenv
         self.ext = ext
         self.grains = grains
         self.minion_id = minion_id
         self.channel = salt.transport.client.AsyncReqChannel.factory(opts)
-        if pillarenv is not None or 'pillarenv' not in self.opts:
+        if pillarenv is not None:
             self.opts['pillarenv'] = pillarenv
-        self.pillar_override = {}
-        if pillar is not None:
-            if isinstance(pillar, dict):
-                self.pillar_override = pillar
-            else:
-                log.error('Pillar data must be a dictionary')
+        self.pillar_override = pillar_override or {}
+        if not isinstance(self.pillar_override, dict):
+            self.pillar_override = {}
+            log.error('Pillar data must be a dictionary')
 
     @tornado.gen.coroutine
     def compile_pillar(self):
@@ -129,21 +130,19 @@ class RemotePillar(object):
     Get the pillar from the master
     '''
     def __init__(self, opts, grains, minion_id, saltenv, ext=None, functions=None,
-                 pillar=None, pillarenv=None):
+                 pillar_override=None, pillarenv=None):
         self.opts = opts
         self.opts['environment'] = saltenv
         self.ext = ext
         self.grains = grains
         self.minion_id = minion_id
         self.channel = salt.transport.Channel.factory(opts)
-        if pillarenv is not None or 'pillarenv' not in self.opts:
+        if pillarenv is not None:
             self.opts['pillarenv'] = pillarenv
-        self.pillar_override = {}
-        if pillar is not None:
-            if isinstance(pillar, dict):
-                self.pillar_override = pillar
-            else:
-                log.error('Pillar data must be a dictionary')
+        self.pillar_override = pillar_override or {}
+        if not isinstance(self.pillar_override, dict):
+            self.pillar_override = {}
+            log.error('Pillar data must be a dictionary')
 
     def compile_pillar(self):
         '''
@@ -175,8 +174,8 @@ class PillarCache(object):
     '''
     Return a cached pillar if it exists, otherwise cache it.
 
-    Pillar caches are structed in two diminensions: minion_id with a dict of saltenvs.
-    Each saltenv contains a pillar dict
+    Pillar caches are structed in two diminensions: minion_id with a dict of
+    saltenvs. Each saltenv contains a pillar dict
 
     Example data structure:
 
@@ -187,7 +186,7 @@ class PillarCache(object):
     '''
     # TODO ABC?
     def __init__(self, opts, grains, minion_id, saltenv, ext=None, functions=None,
-            pillar=None, pillarenv=None):
+            pillar_override=None, pillarenv=None):
         # Yes, we need all of these because we need to route to the Pillar object
         # if we have no cache. This is another refactor target.
 
@@ -197,7 +196,7 @@ class PillarCache(object):
         self.minion_id = minion_id
         self.ext = ext
         self.functions = functions
-        self.pillar = pillar
+        self.pillar_override = pillar_override
         self.pillarenv = pillarenv
 
         if saltenv is None:
@@ -226,13 +225,13 @@ class PillarCache(object):
         '''
         log.debug('Pillar cache getting external pillar with ext: {0}'.format(self.ext))
         fresh_pillar = Pillar(self.opts,
-                                 self.grains,
-                                 self.minion_id,
-                                 self.saltenv,
-                                 ext=self.ext,
-                                 functions=self.functions,
-                                 pillar=self.pillar,
-                                 pillarenv=self.pillarenv)
+                              self.grains,
+                              self.minion_id,
+                              self.saltenv,
+                              ext=self.ext,
+                              functions=self.functions,
+                              pillar_override=self.pillar_override,
+                              pillarenv=self.pillarenv)
         return fresh_pillar.compile_pillar()  # FIXME We are not yet passing pillar_dirs in here
 
     def compile_pillar(self, *args, **kwargs):  # Will likely just be pillar_dirs
@@ -265,9 +264,12 @@ class Pillar(object):
     Read over the pillar top files and render the pillar data
     '''
     def __init__(self, opts, grains, minion_id, saltenv, ext=None, functions=None,
-                 pillar=None, pillarenv=None):
+                 pillar_override=None, pillarenv=None):
         self.minion_id = minion_id
         self.ext = ext
+        if pillarenv is None:
+            if opts.get('pillarenv_from_saltenv', False):
+                opts['pillarenv'] = saltenv
         # Store the file_roots path so we can restore later. Issue 5449
         self.actual_file_roots = opts['file_roots']
         # use the local file client
@@ -303,12 +305,10 @@ class Pillar(object):
 
         self.ext_pillars = salt.loader.pillars(ext_pillar_opts, self.functions)
         self.ignored_pillars = {}
-        self.pillar_override = {}
-        if pillar is not None:
-            if isinstance(pillar, dict):
-                self.pillar_override = pillar
-            else:
-                log.error('Pillar data must be a dictionary')
+        self.pillar_override = pillar_override or {}
+        if not isinstance(self.pillar_override, dict):
+            self.pillar_override = {}
+            log.error('Pillar data must be a dictionary')
 
     def __valid_on_demand_ext_pillar(self, opts):
         '''
@@ -396,13 +396,21 @@ class Pillar(object):
         # Gather initial top files
         try:
             if self.opts['pillarenv']:
-                top = self.client.cache_file(
-                    self.opts['state_top'], self.opts['pillarenv'])
-
-                if top:
+                # If the specified pillarenv is not present in the available
+                # pillar environments, do not cache the pillar top file.
+                if self.opts['pillarenv'] not in self.opts['file_roots']:
+                    log.debug(
+                        'pillarenv \'%s\' not found in the configured pillar '
+                        'environments (%s)',
+                        self.opts['pillarenv'], ', '.join(self.opts['file_roots'])
+                    )
+                else:
                     tops[self.opts['pillarenv']] = [
                             compile_template(
-                                top,
+                                self.client.cache_file(
+                                    self.opts['state_top'],
+                                    self.opts['pillarenv']
+                                    ),
                                 self.rend,
                                 self.opts['renderer'],
                                 self.opts['renderer_blacklist'],
@@ -673,9 +681,11 @@ class Pillar(object):
                                         )
                                 if nstate:
                                     if key:
-                                        nstate = {
-                                            key: nstate
-                                        }
+                                        # If key is x:y, convert it to {x: {y: nstate}}
+                                        for key_fragment in reversed(key.split(":")):
+                                            nstate = {
+                                                key_fragment: nstate
+                                            }
 
                                     state = merge(
                                         state,
@@ -784,7 +794,7 @@ class Pillar(object):
             return pillar, errors
         ext = None
         # Bring in CLI pillar data
-        if self.pillar_override and isinstance(self.pillar_override, dict):
+        if self.pillar_override:
             pillar = merge(pillar,
                            self.pillar_override,
                            self.merge_strategy,
@@ -811,8 +821,16 @@ class Pillar(object):
                                                         pillar_dirs,
                                                         key)
                 except Exception as exc:
-                    errors.append('Failed to load ext_pillar {0}: {1}'.format(
-                        key, exc))
+                    errors.append(
+                        'Failed to load ext_pillar {0}: {1}'.format(
+                            key,
+                            exc.__str__(),
+                        )
+                    )
+                    log.error(
+                        'Execption caught loading ext_pillar \'%s\':\n%s',
+                        key, ''.join(traceback.format_tb(sys.exc_info()[2]))
+                    )
             if ext:
                 pillar = merge(
                     pillar,
@@ -863,15 +881,78 @@ class Pillar(object):
                 log.critical('Pillar render error: {0}'.format(error))
             pillar['_errors'] = errors
 
-        if self.pillar_override and isinstance(self.pillar_override, dict):
+        if self.pillar_override:
             pillar = merge(pillar,
                            self.pillar_override,
                            self.merge_strategy,
                            self.opts.get('renderer', 'yaml'),
                            self.opts.get('pillar_merge_lists', False))
 
+        decrypt_errors = self.decrypt_pillar(pillar)
+        if decrypt_errors:
+            pillar.setdefault('_errors', []).extend(decrypt_errors)
         return pillar
 
+    def decrypt_pillar(self, pillar):
+        '''
+        Decrypt the specified pillar dictionary items, if configured to do so
+        '''
+        errors = []
+        if self.opts.get('decrypt_pillar'):
+            decrypt_pillar = self.opts['decrypt_pillar']
+            if not isinstance(decrypt_pillar, dict):
+                decrypt_pillar = \
+                    salt.utils.repack_dictlist(self.opts['decrypt_pillar'])
+            if not decrypt_pillar:
+                errors.append('decrypt_pillar config option is malformed')
+            for key, rend in six.iteritems(decrypt_pillar):
+                ptr = salt.utils.traverse_dict(
+                    pillar,
+                    key,
+                    default=None,
+                    delimiter=self.opts['decrypt_pillar_delimiter'])
+                if ptr is None:
+                    log.debug('Pillar key %s not present', key)
+                    continue
+                try:
+                    hash(ptr)
+                    immutable = True
+                except TypeError:
+                    immutable = False
+                try:
+                    ret = salt.utils.crypt.decrypt(
+                        ptr,
+                        rend or self.opts['decrypt_pillar_default'],
+                        renderers=self.rend,
+                        opts=self.opts,
+                        valid_rend=self.opts['decrypt_pillar_renderers'])
+                    if immutable:
+                        # Since the key pointed to an immutable type, we need
+                        # to replace it in the pillar dict. First we will find
+                        # the parent, and then we will replace the child key
+                        # with the return data from the renderer.
+                        parent, _, child = key.rpartition(
+                            self.opts['decrypt_pillar_delimiter'])
+                        if not parent:
+                            # key is a top-level key, so the pointer to the
+                            # parent is the pillar dict itself.
+                            ptr = pillar
+                        else:
+                            ptr = salt.utils.traverse_dict(
+                                pillar,
+                                parent,
+                                default=None,
+                                delimiter=self.opts['decrypt_pillar_delimiter'])
+                        if ptr is not None:
+                            ptr[child] = ret
+                except Exception as exc:
+                    msg = 'Failed to decrypt pillar key \'{0}\': {1}'.format(
+                        key, exc
+                    )
+                    errors.append(msg)
+                    log.error(msg, exc_info=True)
+        return errors
+
 
 # TODO: actually migrate from Pillar to AsyncPillar to allow for futures in
 # ext_pillar etc.
