@@ -63,13 +63,6 @@ the mine where it can be easily retrieved by other minions.
     /etc/pki/issued_certs:
       file.directory: []
 
-    /etc/pki/ca.key:
-      x509.private_key_managed:
-        - bits: 4096
-        - backup: True
-        - require:
-          - file: /etc/pki
-
     /etc/pki/ca.crt:
       x509.certificate_managed:
         - signing_private_key: /etc/pki/ca.key
@@ -84,8 +77,12 @@ the mine where it can be easily retrieved by other minions.
         - days_valid: 3650
         - days_remaining: 0
         - backup: True
+        - managed_private_key:
+            name: /etc/pki/ca.key
+            bits: 4096
+            backup: True
         - require:
-          - x509: /etc/pki/ca.key
+          - file: /etc/pki
 
     mine.send:
       module.run:
@@ -142,10 +139,6 @@ This state creates a private key then requests a certificate signed by ca accord
 
 .. code-block:: yaml
 
-    /etc/pki/www.key:
-      x509.private_key_managed:
-        - bits: 4096
-
     /etc/pki/www.crt:
       x509.certificate_managed:
         - ca_server: ca
@@ -154,6 +147,10 @@ This state creates a private key then requests a certificate signed by ca accord
         - CN: www.example.com
         - days_remaining: 30
         - backup: True
+        - managed_private_key:
+            name: /etc/pki/www.key
+            bits: 4096
+            backup: True
 
 '''
 
@@ -171,6 +168,11 @@ import salt.utils
 # Import 3rd-party libs
 import salt.ext.six as six
 
+try:
+    from M2Crypto.RSA import RSAError
+except ImportError:
+    pass
+
 
 def __virtual__():
     '''
@@ -190,7 +192,8 @@ def _revoked_to_list(revs):
     list_ = []
 
     for rev in revs:
-        for rev_name, props in six.iteritems(rev):             # pylint: disable=unused-variable
+        for rev_name, props in six.iteritems(
+                rev):             # pylint: disable=unused-variable
             dict_ = {}
             for prop in props:
                 for propname, val in six.iteritems(prop):
@@ -202,11 +205,52 @@ def _revoked_to_list(revs):
     return list_
 
 
+def _get_file_args(name, **kwargs):
+    valid_file_args = ['user',
+                       'group',
+                       'mode',
+                       'makedirs',
+                       'dir_mode',
+                       'backup',
+                       'create',
+                       'follow_symlinks',
+                       'check_cmd']
+    file_args = {}
+    extra_args = {}
+    for k, v in kwargs.items():
+        if k in valid_file_args:
+            file_args[k] = v
+        else:
+            extra_args[k] = v
+    file_args['name'] = name
+    return file_args, extra_args
+
+
+def _check_private_key(name, bits=2048, passphrase=None,
+                       new=False, overwrite=False):
+    current_bits = 0
+    if os.path.isfile(name):
+        try:
+            current_bits = __salt__['x509.get_private_key_size'](
+                private_key=name, passphrase=passphrase)
+        except salt.exceptions.SaltInvocationError:
+            pass
+        except RSAError:
+            if not overwrite:
+                raise salt.exceptions.CommandExecutionError(
+                    'The provided passphrase cannot decrypt the private key.')
+
+    return current_bits == bits and not new
+
+
 def private_key_managed(name,
                         bits=2048,
+                        passphrase=None,
+                        cipher='aes_128_cbc',
                         new=False,
-                        backup=False,
-                        verbose=True,):
+                        overwrite=False,
+                        verbose=True,
+                        **kwargs):
     '''
     Manage a private key's existence.
 
@@ -216,14 +260,18 @@ def private_key_managed(name,
     bits:
         Key length in bits. Default 2048.
 
+    passphrase:
+        Passphrase for encrypting the private key.
+
+    cipher:
+        Cipher for encrypting the private key.
+
     new:
         Always create a new key. Defaults to False.
-        Combining new with :mod:`prereq <salt.states.requsities.preqreq>` can allow key rotation
-        whenever a new certificiate is generated.
+        Combining new with :mod:`prereq <salt.states.requsities.preqreq>`, or when used as part of a `managed_private_key` can allow key rotation whenever a new certificiate is generated.
 
-    backup:
-        When replacing an existing file, backup the old file on the minion.
-        Default is False.
+    overwrite:
+        Overwrite an existing private key if the provided passphrase cannot decrypt it.
 
     verbose:
         Provide visual feedback on stdout, dots while key is generated.
@@ -231,6 +279,9 @@ def private_key_managed(name,
 
         .. versionadded:: 2016.11.0
 
+    kwargs:
+        Any kwargs supported by file.managed are supported.
+
     Example:
 
     The jinja templating in this example ensures a private key is generated if the file doesn't exist
@@ -247,45 +298,25 @@ def private_key_managed(name,
               - x509: /etc/pki/www.crt
             {%- endif %}
     '''
-    ret = {'name': name, 'changes': {}, 'result': False, 'comment': ''}
-
-    current_bits = 0
-    if os.path.isfile(name):
-        try:
-            current_bits = __salt__['x509.get_private_key_size'](private_key=name)
-            current = "{0} bit private key".format(current_bits)
-        except salt.exceptions.SaltInvocationError:
-            current = '{0} is not a valid Private Key.'.format(name)
+    file_args, kwargs = _get_file_args(name, **kwargs)
+    new_key = False
+    if _check_private_key(
+            name, bits=bits, passphrase=passphrase, new=new, overwrite=overwrite):
+        file_args['contents'] = __salt__['x509.get_pem_entry'](
+            name, pem_type='RSA PRIVATE KEY')
     else:
-        current = '{0} does not exist.'.format(name)
+        new_key = True
+        file_args['contents'] = __salt__['x509.create_private_key'](
+            text=True, bits=bits, passphrase=passphrase, cipher=cipher, verbose=verbose)
 
-    if current_bits == bits and not new:
-        ret['result'] = True
-        ret['comment'] = 'The Private key is already in the correct state'
-        return ret
-
-    ret['changes'] = {
-            'old': current,
-            'new': "{0} bit private key".format(bits)}
-
-    if __opts__['test'] is True:
-        ret['result'] = None
-        ret['comment'] = 'The Private Key "{0}" will be updated.'.format(name)
-        return ret
-
-    if os.path.isfile(name) and backup:
-        bkroot = os.path.join(__opts__['cachedir'], 'file_backup')
-        salt.utils.backup_minion(name, bkroot)
-
-    ret['comment'] = __salt__['x509.create_private_key'](
-        path=name, bits=bits, verbose=verbose)
-    ret['result'] = True
+    ret = __states__['file.managed'](**file_args)
+    if ret['changes'] and new_key:
+        ret['changes'] = 'New private key generated'
 
     return ret
 
 
 def csr_managed(name,
-                backup=False,
                 **kwargs):
     '''
     Manage a Certificate Signing Request
@@ -297,6 +328,9 @@ def csr_managed(name,
         The properties to be added to the certificate request, including items like subject, extensions
         and public key. See above for valid properties.
 
+    kwargs:
+        Any arguments supported by :state:`file.managed <salt.states.file.managed>` are supported.
+
     Example:
 
     .. code-block:: yaml
@@ -310,45 +344,27 @@ def csr_managed(name,
              - L: Salt Lake City
              - keyUsage: 'critical dataEncipherment'
     '''
-    ret = {'name': name, 'changes': {}, 'result': False, 'comment': ''}
+    try:
+        old = __salt__['x509.read_csr'](name)
+    except salt.exceptions.SaltInvocationError:
+        old = '{0} is not a valid csr.'.format(name)
 
-    if os.path.isfile(name):
-        try:
-            current = __salt__['x509.read_csr'](csr=name)
-        except salt.exceptions.SaltInvocationError:
-            current = '{0} is not a valid CSR.'.format(name)
-    else:
-        current = '{0} does not exist.'.format(name)
+    file_args, kwargs = _get_file_args(name, **kwargs)
+    file_args['contents'] = __salt__['x509.create_csr'](text=True, **kwargs)
 
-    new_csr = __salt__['x509.create_csr'](text=True, **kwargs)
-    new = __salt__['x509.read_csr'](csr=new_csr)
-
-    if current == new:
-        ret['result'] = True
-        ret['comment'] = 'The CSR is already in the correct state'
-        return ret
-
-    ret['changes'] = {
-            'old': current,
-            'new': new, }
-
-    if __opts__['test'] is True:
-        ret['result'] = None
-        ret['comment'] = 'The CSR {0} will be updated.'.format(name)
-
-    if os.path.isfile(name) and backup:
-        bkroot = os.path.join(__opts__['cachedir'], 'file_backup')
-        salt.utils.backup_minion(name, bkroot)
-
-    ret['comment'] = __salt__['x509.write_pem'](text=new_csr, path=name, pem_type="CERTIFICATE REQUEST")
-    ret['result'] = True
+    ret = __states__['file.managed'](**file_args)
+    if ret['changes']:
+        new = __salt__['x509.read_csr'](file_args['contents'])
+        if old != new:
+            ret['changes'] = {"Old": old, "New": new}
 
     return ret
 
 
 def certificate_managed(name,
                         days_remaining=90,
-                        backup=False,
+                        managed_private_key=None,
+                        append_certs=None,
                         **kwargs):
     '''
     Manage a Certificate
@@ -360,12 +376,14 @@ def certificate_managed(name,
         The minimum number of days remaining when the certificate should be recreated. Default is 90. A
         value of 0 disables automatic renewal.
 
-    backup:
-        When replacing an existing file, backup the old file on the minion. Default is False.
+    managed_private_key:
+        Manages the private key corresponding to the certificate. All of the arguments supported by :state:`x509.private_key_managed <salt.states.x509.private_key_managed>` are supported. If `name` is not speicified or is the same as the name of the certificate, the private key and certificate will be written together in the same file.
+
+    append_certs:
+        A list of certificates to be appended to the managed file.
 
     kwargs:
-        Any arguments supported by :mod:`x509.create_certificate <salt.modules.x509.create_certificate>`
-        are supported.
+        Any arguments supported by :mod:`x509.create_certificate <salt.modules.x509.create_certificate>` or :state:`file.managed <salt.states.file.managed>` are supported.
 
     Examples:
 
@@ -400,11 +418,44 @@ def certificate_managed(name,
             - backup: True
 
     '''
-    ret = {'name': name, 'changes': {}, 'result': False, 'comment': ''}
-
     if 'path' in kwargs:
         name = kwargs.pop('path')
 
+    file_args, kwargs = _get_file_args(name, **kwargs)
+
+    rotate_private_key = False
+    new_private_key = False
+    if managed_private_key:
+        private_key_args = {
+            'name': name,
+            'new': False,
+            'overwrite': False,
+            'bits': 2048,
+            'passphrase': None,
+            'cipher': 'aes_128_cbc',
+            'verbose': True
+        }
+        private_key_args.update(managed_private_key)
+        kwargs['public_key_passphrase'] = private_key_args['passphrase']
+
+        if private_key_args['new']:
+            rotate_private_key = True
+            private_key_args['new'] = False
+
+        if _check_private_key(private_key_args['name'],
+                              bits=private_key_args['bits'],
+                              passphrase=private_key_args['passphrase'],
+                              new=private_key_args['new'],
+                              overwrite=private_key_args['overwrite']):
+            private_key = __salt__['x509.get_pem_entry'](
+                private_key_args['name'], pem_type='RSA PRIVATE KEY')
+        else:
+            new_private_key = True
+            private_key = __salt__['x509.create_private_key'](text=True, bits=private_key_args['bits'], passphrase=private_key_args[
+                                                              'passphrase'], cipher=private_key_args['cipher'], verbose=private_key_args['verbose'])
+
+        kwargs['public_key'] = private_key
+
     current_days_remaining = 0
     current_comp = {}
 
@@ -418,7 +469,7 @@ def certificate_managed(name,
                     try:
                         current_comp['X509v3 Extensions']['authorityKeyIdentifier'] = (
                             re.sub(r'serial:([0-9A-F]{2}:)*[0-9A-F]{2}', 'serial:--',
-                                current_comp['X509v3 Extensions']['authorityKeyIdentifier']))
+                                   current_comp['X509v3 Extensions']['authorityKeyIdentifier']))
                     except KeyError:
                         pass
             current_comp.pop('Not Before')
@@ -427,8 +478,8 @@ def certificate_managed(name,
             current_comp.pop('SHA-256 Finger Print')
             current_notafter = current_comp.pop('Not After')
             current_days_remaining = (
-                    datetime.datetime.strptime(current_notafter, '%Y-%m-%d %H:%M:%S') -
-                    datetime.datetime.now()).days
+                datetime.datetime.strptime(current_notafter, '%Y-%m-%d %H:%M:%S') -
+                datetime.datetime.now()).days
             if days_remaining == 0:
                 days_remaining = current_days_remaining - 1
         except salt.exceptions.SaltInvocationError:
@@ -437,7 +488,8 @@ def certificate_managed(name,
         current = '{0} does not exist.'.format(name)
 
     if 'ca_server' in kwargs and 'signing_policy' not in kwargs:
-        raise salt.exceptions.SaltInvocationError('signing_policy must be specified if ca_server is.')
+        raise salt.exceptions.SaltInvocationError(
+            'signing_policy must be specified if ca_server is.')
 
     new = __salt__['x509.create_certificate'](testrun=True, **kwargs)
 
@@ -450,7 +502,7 @@ def certificate_managed(name,
                 try:
                     new_comp['X509v3 Extensions']['authorityKeyIdentifier'] = (
                         re.sub(r'serial:([0-9A-F]{2}:)*[0-9A-F]{2}', 'serial:--',
-                            new_comp['X509v3 Extensions']['authorityKeyIdentifier']))
+                               new_comp['X509v3 Extensions']['authorityKeyIdentifier']))
                 except KeyError:
                     pass
         new_comp.pop('Not Before')
@@ -462,41 +514,72 @@ def certificate_managed(name,
     else:
         new_comp = new
 
+    new_certificate = False
     if (current_comp == new_comp and
             current_days_remaining > days_remaining and
             __salt__['x509.verify_signature'](name, new_issuer_public_key)):
-        ret['result'] = True
-        ret['comment'] = 'The certificate is already in the correct state'
-        return ret
+        certificate = __salt__['x509.get_pem_entry'](
+            name, pem_type='CERTIFICATE')
+    else:
+        if rotate_private_key and not new_private_key:
+            new_private_key = True
+            private_key = __salt__['x509.create_private_key'](
+                text=True, bits=private_key_args['bits'], verbose=private_key_args['verbose'])
+            kwargs['public_key'] = private_key
+        new_certificate = True
+        certificate = __salt__['x509.create_certificate'](text=True, **kwargs)
 
-    ret['changes'] = {
-            'old': current,
-            'new': new, }
+    file_args['contents'] = ''
+    private_ret = {}
+    if managed_private_key:
+        if private_key_args['name'] == name:
+            file_args['contents'] = private_key
+        else:
+            private_file_args = copy.deepcopy(file_args)
+            unique_private_file_args, _ = _get_file_args(**private_key_args)
+            private_file_args.update(unique_private_file_args)
+            private_file_args['contents'] = private_key
+            private_ret = __states__['file.managed'](**private_file_args)
+            if not private_ret['result']:
+                return private_ret
 
-    if __opts__['test'] is True:
-        ret['result'] = None
-        ret['comment'] = 'The certificate {0} will be updated.'.format(name)
-        return ret
+    file_args['contents'] += certificate
 
-    if os.path.isfile(name) and backup:
-        bkroot = os.path.join(__opts__['cachedir'], 'file_backup')
-        salt.utils.backup_minion(name, bkroot)
+    if not append_certs:
+        append_certs = []
+    for append_cert in append_certs:
+        file_args[
+            'contents'] += __salt__['x509.get_pem_entry'](append_cert, pem_type='CERTIFICATE')
 
-    ret['comment'] = __salt__['x509.create_certificate'](path=name, **kwargs)
-    ret['result'] = True
+    file_args['show_changes'] = False
+    ret = __states__['file.managed'](**file_args)
+
+    if ret['changes']:
+        ret['changes'] = {'Certificate': ret['changes']}
+    else:
+        ret['changes'] = {}
+    if private_ret and private_ret['changes']:
+        ret['changes']['Private Key'] = private_ret['changes']
+    if new_private_key:
+        ret['changes']['Private Key'] = 'New private key generated'
+    if new_certificate:
+        ret['changes']['Certificate'] = {
+            'Old': current,
+            'New': __salt__['x509.read_certificate'](certificate=certificate)}
 
     return ret
 
 
 def crl_managed(name,
                 signing_private_key,
+                signing_private_key_passphrase=None,
                 signing_cert=None,
                 revoked=None,
                 days_valid=100,
                 digest="",
                 days_remaining=30,
                 include_expired=False,
-                backup=False,):
+                **kwargs):
     '''
     Manage a Certificate Revocation List
 
@@ -507,6 +590,9 @@ def crl_managed(name,
         The private key that will be used to sign this crl. This is
         usually your CA's private key.
 
+    signing_private_key_passphrase:
+        Passphrase to decrypt the private key.
+
     signing_cert:
         The certificate of the authority that will be used to sign this crl.
         This is usually your CA's certificate.
@@ -530,8 +616,8 @@ def crl_managed(name,
     include_expired:
         Include expired certificates in the CRL. Default is ``False``.
 
-    backup:
-        When replacing an existing file, backup the old file on the minion. Default is False.
+    kwargs:
+        Any arguments supported by :state:`file.managed <salt.states.file.managed>` are supported.
 
     Example:
 
@@ -552,8 +638,6 @@ def crl_managed(name,
                 - revocation_date: 2015-02-25 00:00:00
                 - reason: cessationOfOperation
     '''
-    ret = {'name': name, 'changes': {}, 'result': False, 'comment': ''}
-
     if revoked is None:
         revoked = []
 
@@ -569,8 +653,8 @@ def crl_managed(name,
             current_comp.pop('Last Update')
             current_notafter = current_comp.pop('Next Update')
             current_days_remaining = (
-                    datetime.datetime.strptime(current_notafter, '%Y-%m-%d %H:%M:%S') -
-                    datetime.datetime.now()).days
+                datetime.datetime.strptime(current_notafter, '%Y-%m-%d %H:%M:%S') -
+                datetime.datetime.now()).days
             if days_remaining == 0:
                 days_remaining = current_days_remaining - 1
         except salt.exceptions.SaltInvocationError:
@@ -578,44 +662,36 @@ def crl_managed(name,
     else:
         current = '{0} does not exist.'.format(name)
 
-    new_crl = __salt__['x509.create_crl'](text=True, signing_private_key=signing_private_key,
-            signing_cert=signing_cert, revoked=revoked, days_valid=days_valid, digest=digest, include_expired=include_expired)
+    new_crl = __salt__['x509.create_crl'](text=True, signing_private_key=signing_private_key, signing_private_key_passphrase=signing_private_key_passphrase,
+                                          signing_cert=signing_cert, revoked=revoked, days_valid=days_valid, digest=digest, include_expired=include_expired)
 
     new = __salt__['x509.read_crl'](crl=new_crl)
     new_comp = new.copy()
     new_comp.pop('Last Update')
     new_comp.pop('Next Update')
 
+    file_args, kwargs = _get_file_args(name, **kwargs)
+    new_crl_created = False
     if (current_comp == new_comp and
             current_days_remaining > days_remaining and
             __salt__['x509.verify_crl'](name, signing_cert)):
+        file_args['contents'] = __salt__[
+            'x509.get_pem_entry'](name, pem_type='X509 CRL')
+    else:
+        new_crl_created = True
+        file_args['contents'] = new_crl
 
-        ret['result'] = True
-        ret['comment'] = 'The crl is already in the correct state'
-        return ret
-
-    ret['changes'] = {
-            'old': current,
-            'new': new, }
-
-    if __opts__['test'] is True:
-        ret['result'] = None
-        ret['comment'] = 'The crl {0} will be updated.'.format(name)
-        return ret
-
-    if os.path.isfile(name) and backup:
-        bkroot = os.path.join(__opts__['cachedir'], 'file_backup')
-        salt.utils.backup_minion(name, bkroot)
-
-    ret['comment'] = __salt__['x509.write_pem'](text=new_crl, path=name, pem_type='X509 CRL')
-    ret['result'] = True
-
+    ret = __states__['file.managed'](**file_args)
+    if new_crl_created:
+        ret['changes'] = {'Old': current, 'New': __salt__[
+            'x509.read_crl'](crl=new_crl)}
     return ret
 
 
 def pem_managed(name,
                 text,
-                backup=False):
+                backup=False,
+                **kwargs):
     '''
     Manage the contents of a PEM file directly with the content in text, ensuring correct formatting.
 
@@ -625,37 +701,10 @@ def pem_managed(name,
     text:
         The PEM formatted text to write.
 
-    backup:
-        When replacing an existing file, backup the old file on the minion. Default is False.
+    kwargs:
+        Any arguments supported by :state:`file.managed <salt.states.file.managed>` are supported.
     '''
-    ret = {'name': name, 'changes': {}, 'result': False, 'comment': ''}
+    file_args, kwargs = _get_file_args(name, **kwargs)
+    file_args['contents'] = __salt__['x509.get_pem_entry'](text=text)
 
-    new = __salt__['x509.get_pem_entry'](text=text)
-
-    try:
-        with salt.utils.fopen(name) as fp_:
-            current = fp_.read()
-    except (OSError, IOError):
-        current = '{0} does not exist or is unreadable'.format(name)
-
-    if new == current:
-        ret['result'] = True
-        ret['comment'] = 'The file is already in the correct state'
-        return ret
-
-    ret['changes']['new'] = new
-    ret['changes']['old'] = current
-
-    if __opts__['test'] is True:
-        ret['result'] = None
-        ret['comment'] = 'The file {0} will be updated.'.format(name)
-        return ret
-
-    if os.path.isfile(name) and backup:
-        bkroot = os.path.join(__opts__['cachedir'], 'file_backup')
-        salt.utils.backup_minion(name, bkroot)
-
-    ret['comment'] = __salt__['x509.write_pem'](text=text, path=name)
-    ret['result'] = True
-
-    return ret
+    return __states__['file.managed'](**file_args)
